/******************************************************************************
FILE:               O2CI8Object.h
LANGUAGE:           Objective-C
SYSTEM:             None
USER-INTERFACE:     None
DESCRIPTION
    $Header$
    This is the definition of the O2CI8Object class.
    The class O2CI8Object encapsulate OCI handles. 
	It's the root class of all O2CI8 encapsulating classes.
	It's an abstract class. You can instanciate only subclasses.

		OCI  = Oracle8 C Interface.
		O2CI8 = Oracle8 Objective C Interface.

AUTHORS
    <PJB>  Pascal J. Bourguignon  <pjb@imaginet.fr>
MODIFICATIONS
    $Log$
    1999/07/27 Creation.
BUGS
    - It's not implemented yet.
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#ifndef __O2CI8Object_h__
#define __O2CI8Object_h__
#include <Foundation/Foundation.h>
#include <oci.h>
@class O2CI8Error; // forward reference.

extern NSString* O2CI8Object_eAbstractClass;
extern NSString* O2CI8Object_eInvalidHandle;
extern NSString* O2CI8Object_eInvalidState;
extern NSString* O2CI8Object_eInvalidParameter;



@interface O2CI8Object:NSObject
{
	dvoid*              handle;
	ub4                 handleType;
	BOOL                handleIsValid;
	NSMutableArray*     errorMessages; // automatically managed by this class.
	O2CI8Error*          error;         // Subclasses must initialize it if needed.
}

// NSObject methods:

    -(id)init;
		/*
			RETURN:     self, or nil when a problem occur with [super init].
			IPOST:      self!=nil ==> ![self handleIsValid], error==nil.
		*/

    -(void)dealloc;


// O2CI8Object methods:

    -(id)initWithHandle:(dvoid*)anHandle type:(ub4)anHandleType;
		/*
			IPRE:       anHandle & anHandleType describe a valid OCI handle.

			NOTE:       Usually, subclasses overriding this method have 
						a more restrictive PRE condition, such as restricting 
						anHandleType to a precise value.

			RETURN:     self, or nil when a problem occur with [super init].

		    POST:       self!=nil ==> [self handleIsValid].

			NOTE:       Client must take care to provide a valid handle/type.

			NOTE:       Subclasses may need more setup, in particular, they
						must allocate the error attribute if they need to
						report OCI errors.
		*/


	-(BOOL)handleIsValid;
		/*
			RETURN:     whether the handle is valid, ie. has correctly
						been allocated by OCI.
		*/


	-(dvoid*)handle;
		/*
			PRE:        [self handleIsValid].
			RAISE:      O2CI8Object_eInvalidHandle when ![self handleIsValid].
			RETURN:     the handle.
		*/

	-(ub4)handleType;
		/*
			PRE:        [self handleIsValid].
			RAISE:      O2CI8Object_eInvalidHandle when ![self handleIsValid].
			RETURN:     the handleType.
		*/


// Error Management methods:  

	/*
		The principle is that each O2CI8Object may have its own
		O2CI8Error instance in which its own error are reported (it
		passes [error handle] to OCI calls). O2CI8Error instances only
		encapsulate error handles that are used by the OCI library to
		report error messages. Each O2CI8Object can process its own
		errors with the following methods:

			- processErrorCode:from:mayRaise:, and
			- processErrorCode:from: (which may always raise).

		For now, only an invalid handle may raise an exception. These
		two methods return a boolean indicating success (either
		OCI_SUCCESS or OCI_SUCCESS_WITH_INFO. The error or information
		message(s) are stored in a array returned by
		errorMessages. Even in case of success, the client may use
		this later method to retrieve the information messages.
	*/


	-(BOOL)processErrorCode:(sword)ociResult
			from:(NSString*)location mayRaise:(BOOL)mayRaise;
		/*
			DO:         Logs an error message, when ociResult & error
						correspond to an error. In addition, store the
						error message (or messages when there are several)
						into an array for later retrieval and reporting.

			POST:       (ociResult==OCI_SUCCESS)
							<==> [[self errorMessages] count]==0

			RAISE:      O2CI8Object_eInvalidHandle 
							when mayRaise&&(ociResult==OCI_INVALID_HANDLE).

			RETURN:     (ociResult==OCI_SUCCESS)
						||(ociResult==OCI_SUCCESS_WITH_INFO).
		*/

	-(BOOL)processErrorCode:(sword)ociResult from:(NSString*)location;
		/*
			DO:         [self processErrorCode:ociResult 
								from:location mayRaise:YES].
		*/

	-(NSArray*)errorMessages;
		/*
			RETURN:     an autoreleased array containing 0 or more NSString
						with the error messages of the last OCI call done
						by this instance. No string in the array is not
						terminated by a new-line, and normaly would not
						contain any new-line. The firt string is set
						by O2CI8Object, and the others come from OCI.
						The first string contains the location given to
						processErrorCode:... and the description of self.

			NOTE:       [[o errorMessages]count]==0 <=> ociResult was OCI_SUCCESS.
		*/


// Generic handle attributes:

	-(NSString*)functionName;
		/*
			PRE:        [self handleType] is in {OCI_HTYPE_ENV,OCI_HTYPE_SERVER,
												 OCI_HTYPE_STMT,OCI_HTYPE_DEFINE}.
						Meaning, self is a kind of O2CI8Environment, 
						O2CI8ServerConnection, O2CI8Statement or O2CI8Define.

			RETURN:     The name of the last OCI operation on this handle. 
		*/

// Attribute management methods:

	-(BOOL)setString:(NSString*)value 
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)setDVoidP:(dvoid*)value
			attribute:(ub4)attribute from:(NSString*)location;


	-(BOOL)getString:(NSString**)value 
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)getDVoidP:(dvoid**)value
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)getWord:(sword*)value
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)getUb4:(ub4*)value
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)getUb2:(ub2*)value
			attribute:(ub4)attribute from:(NSString*)location;

	-(BOOL)getUb1:(ub1*)value
			attribute:(ub4)attribute from:(NSString*)location;



// Protected methods:

	-(NSBundle*)bundle;
		/*
			RETURN:     The bundle from where this class was loaded.

			NOTE:       We don't use Oracle8Utilities_Bundle() to be independant
						of the `client' Oracle8Adaptor layer.
		*/


@end //O2CI8Object;

#endif /*__O2CI8Object_h__*/

/*** O2CI8Object.h -- 1999/07/27 06:35:53 -- <PJB> ***/

