/******************************************************************************
FILE:               O2CI8Object.m
LANGUAGE:           Objective-C
SYSTEM:             None
USER-INTERFACE:     None
DESCRIPTION
    $Header$
    This is the implementation of the O2CI8Object class.
    The class O2CI8Object encapsulate OCI handles. It's the root class
	of all OCI encapsulating classes.

AUTHORS
    <PJB>  Pascal J. Bourguignon  <pjb@imaginet.fr>
MODIFICATIONS
    $Log$
    1999/07/27 Creation.
BUGS
    - It's not implemented yet.
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#include <O2CI8Object.h>
#include <O2CI8Error.h>
#include <Oracle8Utilities.h>


DECLARE_SYMBOL(O2CI8Object_eAbstractClass)
DECLARE_SYMBOL(O2CI8Object_eInvalidHandle)
DECLARE_SYMBOL(O2CI8Object_eInvalidState)
DECLARE_SYMBOL(O2CI8Object_eInvalidParameter)


@implementation O2CI8Object

    static const char*     rcsid="$Id";


    -(id)init
    {
        self=[super init];
        if(self!=nil){
            handle=(dvoid*)0;
			handleType=0;
			handleIsValid=NO;
			error=nil; // subclasses will allocate it if needed.
			errorMessages=nil;
        }
        return(self);
    }//init;


    -(void)dealloc
    {
		if(handleIsValid){
			// SEE: freeing sub-handles should have been done by subclasses.
			[error release];
			[self processErrorCode:OCIHandleFree(handle,handleType)
					from:CURRENT_METHOD_AND(@"/OCIHandleFree") mayRaise:NO];
			[errorMessages release];
		}
        [super dealloc];
    }//dealloc;


// O2CI8Object methods:

    -(id)initWithHandle:(dvoid*)anHandle type:(ub4)anHandleType
    {
        self=[self init];
        if(self!=nil){
            handle=anHandle;
            handleType=anHandleType;
            handleIsValid=YES;
			errorMessages=nil;
			error=nil;
        }
        return(self);
    }//initWithHandle:;


	-(BOOL)handleIsValid
	{
		return(handleIsValid);
	}//handleIsValid;


	-(dvoid*)handle
	{
		if(!handleIsValid){
			[NSException raise:O2CI8Object_eInvalidHandle format:@""];
		}
		return(handle);
	}//handle;


	-(ub4)handleType
	{
		if(!handleIsValid){
			[NSException raise:O2CI8Object_eInvalidHandle format:@""];
		}
		return(handleType);
	}//handleType;


	-(BOOL)processErrorCode:(sword)ociResult 
			from:(NSString*)location mayRaise:(BOOL)mayRaise
	{
		NSString* qualifier=nil;
		NSString* message;

		if(errorMessages!=nil){
			[errorMessages removeAllObjects];
		}

		switch(ociResult){
		case OCI_SUCCESS:
			return(YES);

		case OCI_SUCCESS_WITH_INFO:
			qualifier=NSLocalizedStringFromTableInBundle(@"Oracle8 information",\
					@"O2CI8",[self bundle],@"OCI message qualifier.");
			// fall thru;

		case OCI_ERROR:
			if(qualifier==nil){
				qualifier=NSLocalizedStringFromTableInBundle(@"Oracle8 error",\
						@"O2CI8",[self bundle],@"OCI message qualifier.");
			}
			if(error==nil){
				message=[NSString stringWithFormat:
							NSLocalizedStringFromTableInBundle(@"[%@ in %@] %@: %@",@"O2CI8",[self bundle],@"error message format. [1]: description of error generating object. [2]: location of error generating code. [3]: OCI message qualifier, or OCI result code. [4]: OCI error message."),
							[self description],location,qualifier,
							NSLocalizedStringFromTableInBundle(@"(no error handle).",@"O2CI8",[self bundle],@"error message when there is no error handle to report it.")];
				if(errorMessages==nil){
					errorMessages=[[NSMutableArray alloc]initWithCapacity:2];
				}
				[errorMessages addObject:message];
			    NSLog(@"%@\n",message);
			}else{
				ub4  i;
				sb4  oraCode=0;
				NSString* line;

				message=[NSString stringWithFormat:
							NSLocalizedStringFromTableInBundle(@"[%@ in %@/%@] %@:",@"O2CI8",[self bundle],@"error message format. [1]: description of error generating object. [2]: location of error generating code. [3]: Name of the OCI function where the error comes from. [4]: OCI message qualifier, or OCI result code. (the OCI error messages will follow in other strings)."),
							[self description],location,
							[self functionName],qualifier];
				if(errorMessages==nil){
					errorMessages=[[NSMutableArray alloc]initWithCapacity:2];
				}
				[errorMessages addObject:message];
				i=1;
				while(nil!=(line=
						[error getMessageAndOracle8Code:&oraCode atIndex:i])){
					[errorMessages addObject:line];
					NSLog(@"%@ #%d %@\n",message,i,line);
					i++;
				}
			}
			return(ociResult==OCI_SUCCESS_WITH_INFO);

		default:
			switch(ociResult){
			case OCI_NO_DATA:
				qualifier=NSLocalizedStringFromTableInBundle(\
					@"OCI_NO_DATA",@"O2CI8",[self bundle],@"OCI Result Code.");
				break;

			case OCI_INVALID_HANDLE:
				qualifier=NSLocalizedStringFromTableInBundle(\
					@"OCI_INVALID_HANDLE",@"O2CI8",[self bundle],@"OCI Result Code.");
				break;

			case OCI_NEED_DATA:
				qualifier=NSLocalizedStringFromTableInBundle(\
					@"OCI_NEED_DATA",@"O2CI8",[self bundle],@"OCI Result Code.");
				break;

			case OCI_STILL_EXECUTING:
				qualifier=NSLocalizedStringFromTableInBundle(\
					@"OCI_STILL_EXECUTING",@"O2CI8",[self bundle],@"OCI Result Code.");
				break;
				
			case OCI_CONTINUE:
				qualifier=NSLocalizedStringFromTableInBundle(\
					@"OCI_CONTINUE",@"O2CI8",[self bundle],@"OCI Result Code.");
				break;

			default:
				qualifier=[NSString stringWithFormat:
					NSLocalizedStringFromTableInBundle(\
					@"Unknown OCI Result Code #%d",@"O2CI8",[self bundle],\
					@"Unknown OCI Result Code. [1]: value of the result code."),
					ociResult];
				break;
			}

			message=[NSString stringWithFormat:
							NSLocalizedStringFromTableInBundle(@"[%@ in %@] OCI Error: %@",@"O2CI8",[self bundle],@"error message format. [1]: description of error generating object. [2]: location of error generating code. [3]: OCI result code label."),
							[self description],location,qualifier];
			if(errorMessages==nil){
				errorMessages=[[NSMutableArray alloc]initWithCapacity:2];
			}
			[errorMessages addObject:message];
			NSLog(@"%@\n",message);
			
			if((ociResult==OCI_INVALID_HANDLE)&&(mayRaise)){
				[NSException raise:O2CI8Object_eInvalidHandle format:@"%@",message];
			}
			return(NO);
		}
	}//processErrorCode:from:mayRaise:;


	-(BOOL)processErrorCode:(sword)ociResult from:(NSString*)location
	{
		return([self processErrorCode:ociResult from:location mayRaise:YES]);
	}//processErrorCode:from:;



	-(NSArray*)errorMessages
	{
		if(errorMessages==nil){
			return([NSArray array]);
		}else{
			return([[errorMessages copy]autorelease]);
		}
	}//errorMessages;


// Generic handle attributes:

	-(NSString*)functionName
	{
		ub4 functionCode;

		static NSString*       FunctionCodeToName[81];
		static const int       FunctionCodeToName_Last=80;
		static BOOL            FunctionCodeToName_isInitialized=FALSE;

		if(!FunctionCodeToName_isInitialized){
			FunctionCodeToName[0]=@"unused"; // SEE: Localization.
			FunctionCodeToName[1]=@"OCIInitialize";
			FunctionCodeToName[2]=@"OCIHandleAlloc";
			FunctionCodeToName[3]=@"OCIHandleFree";
			FunctionCodeToName[4]=@"OCIDescriptorAlloc";
			FunctionCodeToName[5]=@"OCIDescriptorFree";
			FunctionCodeToName[6]=@"OCIEnvInit";
			FunctionCodeToName[7]=@"OCIServerAttach";
			FunctionCodeToName[8]=@"OCIServerDetach";
			FunctionCodeToName[9]=@"unused";  // SEE: Localization.
			FunctionCodeToName[10]=@"OCISessionBegin";
			FunctionCodeToName[11]=@"OCISessionEnd";
			FunctionCodeToName[12]=@"OCIPasswordChange";
			FunctionCodeToName[13]=@"OCIStmtPrepare";
			FunctionCodeToName[14]=@"unused"; // SEE: Localization.
			FunctionCodeToName[15]=@"unused"; // SEE: Localization.
			FunctionCodeToName[16]=@"unused"; // SEE: Localization.
			FunctionCodeToName[17]=@"OCIBindDynamic";
			FunctionCodeToName[18]=@"OCIBindObject";
			FunctionCodeToName[19]=@"unused"; // SEE: Localization.
			FunctionCodeToName[20]=@"OCIBindArrayOfStruct";
			FunctionCodeToName[21]=@"OCIStmtExecute";
			FunctionCodeToName[22]=@"unused"; // SEE: Localization.
			FunctionCodeToName[23]=@"unused"; // SEE: Localization.
			FunctionCodeToName[24]=@"unused"; // SEE: Localization.
			FunctionCodeToName[25]=@"OCIDefineObject";
			FunctionCodeToName[26]=@"OCIDefineDynamic";
			FunctionCodeToName[27]=@"OCIDefineArrayOfStruct";
			FunctionCodeToName[28]=@"OCIStmtFetch";
			FunctionCodeToName[29]=@"OCIStmtGetBindInfo";
			FunctionCodeToName[30]=@"unused"; // SEE: Localization.
			FunctionCodeToName[31]=@"unused"; // SEE: Localization.
			FunctionCodeToName[32]=@"OCIDescribeAny";
			FunctionCodeToName[33]=@"OCITransStart";
			FunctionCodeToName[34]=@"OCITransDetach";
			FunctionCodeToName[35]=@"OCITransCommit";
			FunctionCodeToName[36]=@"unused"; // SEE: Localization.
			FunctionCodeToName[37]=@"OCIErrorGet";
			FunctionCodeToName[38]=@"OCILobFileOpen";
			FunctionCodeToName[39]=@"OCILobFileClose";
			FunctionCodeToName[40]=@"was LOBCREATEFILE, unused"; // SEE: Localization.
			FunctionCodeToName[41]=@"was OCILobFileDelete, unused"; // SEE: Localization.
			FunctionCodeToName[42]=@"OCILobCopy";
			FunctionCodeToName[43]=@"OCILobAppend";
			FunctionCodeToName[44]=@"OCILobErase";
			FunctionCodeToName[45]=@"OCILobGetLength";
			FunctionCodeToName[46]=@"OCILobTrim";
			FunctionCodeToName[47]=@"OCILobRead";
			FunctionCodeToName[48]=@"OCILobWrite";
			FunctionCodeToName[49]=@"unused";
			FunctionCodeToName[50]=@"OCIBreak";
			FunctionCodeToName[51]=@"OCIServerVersion";
			FunctionCodeToName[52]=@"unused"; // SEE: Localization.
			FunctionCodeToName[53]=@"unused"; // SEE: Localization.
			FunctionCodeToName[54]=@"OCIAttrGet";
			FunctionCodeToName[55]=@"OCIAttrSet";
			FunctionCodeToName[56]=@"OCIParamSet";
			FunctionCodeToName[57]=@"OCIParamGet";
			FunctionCodeToName[58]=@"OCIStmtGetPieceInfo";
			FunctionCodeToName[59]=@"OCILdaToSvcCtx";
			FunctionCodeToName[60]=@"unused"; // SEE: Localization.
			FunctionCodeToName[61]=@"OCIStmtSetPieceInfo";
			FunctionCodeToName[62]=@"OCITransForget";
			FunctionCodeToName[63]=@"OCITransPrepare";
			FunctionCodeToName[64]=@"OCITransRollback";
			FunctionCodeToName[65]=@"OCIDefineByPos";
			FunctionCodeToName[66]=@"OCIBindByPos";
			FunctionCodeToName[67]=@"OCIBindByName";
			FunctionCodeToName[68]=@"OCILobAssign";
			FunctionCodeToName[69]=@"OCILobIsEqual";
			FunctionCodeToName[70]=@"OCILobLocatorIsInit";
			FunctionCodeToName[71]=@"OCILobEnableBuffering";
			FunctionCodeToName[72]=@"OCILobCharSetID";
			FunctionCodeToName[73]=@"OCILobCharSetForm";
			FunctionCodeToName[74]=@"OCILobFileSetName";
			FunctionCodeToName[75]=@"OCILobFileGetName";
			FunctionCodeToName[76]=@"OCILogon";
			FunctionCodeToName[77]=@"OCILogoff";
			FunctionCodeToName[78]=@"OCILobDisableBuffering";
			FunctionCodeToName[79]=@"OCILobFlushBuffer";
			FunctionCodeToName[80]=@"OCILobLoadFromFile";
			FunctionCodeToName_isInitialized=YES;
		}

		switch(handleType){
		case OCI_HTYPE_ENV:
		case OCI_HTYPE_SERVER:
		case OCI_HTYPE_STMT:
		case OCI_HTYPE_DEFINE:
			if([self getUb4:&functionCode attribute:OCI_ATTR_FNCODE 
					from:CURRENT_METHOD_AND(@"/OCIAttrGet]")]){
				if((0<functionCode)&&(functionCode<=FunctionCodeToName_Last)){
					return(FunctionCodeToName[functionCode]);
				}
			}
		default:
			break;
		}
		return(@"unknown"); // SEE: Localization.
	}//functionName;



// Attribute management methods:

	-(BOOL)setString:(NSString*)value attribute:(ub4)attribute 
			from:(NSString*)location
	{
		sword ociResult;
		if(value==nil){
			ociResult=OCIAttrSet(handle,handleType,(dvoid*)0,0,
							attribute,[error handle]);
		}else{
			/* We have no garantee that the buffer won't be touched */
			text* buffer;
			ub4   length=[value cStringLength];
			buffer=(text*)malloc(sizeof(text)*(length+1));
			[value getCString:buffer maxLength:length];

			ociResult=OCIAttrSet(handle,handleType,
							(dvoid*)buffer,length,attribute,[error handle]);

			free(buffer);
		}
		return([self processErrorCode:ociResult from:location]);
	}//setString:attribute:from:;


	-(BOOL)setDVoidP:(dvoid*)value
			attribute:(ub4)attribute from:(NSString*)location
	{
		return([self processErrorCode:
				OCIAttrSet(handle,handleType,value,0,attribute,[error handle])
			from:location]);
	}//setDVoidP:attribute:from:;



	-(BOOL)getString:(NSString**)value
			attribute:(ub4)attribute from:(NSString*)location
	{
		sword    ociResult;
		ub4      stringLength;
		text*    string;
		ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)&string,(ub4*)&stringLength,
							attribute,[error handle]);
		if(ociResult==OCI_SUCCESS){
			(*value)=[NSString stringWithCString:string length:stringLength];
		}else{
			(*value)=nil;
		}
		return([self processErrorCode:ociResult from:location]);
	}//getString:attribute:from:;


	-(BOOL)getDVoidP:(dvoid**)value
			attribute:(ub4)attribute from:(NSString*)location
	{
		sword    ociResult;
		dvoid*   dummy;
		if(value==(dvoid**)0){
			ociResult=OCIAttrGet(handle,handleType,(dvoid*)&dummy,(ub4*)0,
							attribute,[error handle]);
		}else{
			ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)value,(ub4*)0,
							attribute,[error handle]);
		}
		return([self processErrorCode:ociResult from:location]);
	}//getDVoidP:attribute:from:;



	-(BOOL)getWord:(sword*)value
			attribute:(ub4)attribute from:(NSString*)location
	{
		sword ociResult;
		sword dummy;
		if(value==(sword*)0){
			ociResult=OCIAttrGet(handle,handleType,(dvoid*)&dummy,(ub4*)0,
							attribute,[error handle]);
		}else{
			ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)value,(ub4*)0,
							attribute,[error handle]);
		}
		return([self processErrorCode:ociResult from:location]);
	}//getWord:attribute:from:;



	-(BOOL)getUb4:(ub4*)value attribute:(ub4)attribute from:(NSString*)location
	{
		sword ociResult;
		ub4   dummy;
		if(value==(ub4*)0){
			ociResult=OCIAttrGet(handle,handleType,(dvoid*)&dummy,(ub4*)0,
							attribute,[error handle]);
		}else{
			ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)value,(ub4*)0,
							attribute,[error handle]);
		}
		return([self processErrorCode:ociResult from:location]);
	}//getUb4:attribute:from:;



	-(BOOL)getUb2:(ub2*)value attribute:(ub4)attribute from:(NSString*)location
	{
		sword ociResult;
		ub2   dummy;
		if(value==(ub2*)0){
			ociResult=OCIAttrGet(handle,handleType,(dvoid*)&dummy,(ub4*)0,
							attribute,[error handle]);
		}else{
			ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)value,(ub4*)0,
							attribute,[error handle]);
		}
		return([self processErrorCode:ociResult from:location]);
	}//getUb2:attribute:from:;


	-(BOOL)getUb1:(ub1*)value attribute:(ub4)attribute from:(NSString*)location
	{
		sword ociResult;
		ub1   dummy;
		if(value==(ub1*)0){
			ociResult=OCIAttrGet(handle,handleType,(dvoid*)&dummy,(ub4*)0,
							attribute,[error handle]);
		}else{
			ociResult=OCIAttrGet(handle,handleType,
							(dvoid*)value,(ub4*)0,
							attribute,[error handle]);
		}
		return([self processErrorCode:ociResult from:location]);
	}//getUb1:attribute:from:;





// Protected methods:

	-(NSBundle*)bundle
	{
		static NSBundle* ourBundle=nil;
		if(ourBundle==nil){
			ourBundle=[[NSBundle bundleForClass:[self class]]retain];
	//	NSLog(@"-[O2CI8Object bundle]: bundlePath='%@'; resourcePath='%@'.\n", 
	//		[ourBundle bundlePath],[ourBundle resourcePath]);
		}
		return(ourBundle);
	}//bundle;


@end //O2CI8Object;

/*** O2CI8Object.m -- 1999/07/27 06:35:53 -- <PJB> ***/


