/******************************************************************************
FILE:               O2CI8ServiceContext.h
LANGUAGE:           Objective-C
SYSTEM:             None
USER-INTERFACE:     None
DESCRIPTION
    $Header$
    This is the definition of the O2CI8ServiceContext class.
    The class O2CI8ServiceContext encapsulate a service context handle.
	A service context handle defines attributes that determine the 
	operational context for OCI calls to a server. 
	OCIServiceContext instances allocate and initialize the service 
	context handle with OCIHandleAlloc().

	The service context contains three additional handles that 
	represent a server connection, a user session, and a transaction.

	There is exactly one transaction associated to each service context.
	That may be an implicit transaction, or an explicit one.


AUTHORS
    <PJB>  Pascal J. Bourguignon  <pjb@imaginet.fr>
MODIFICATIONS
    $Log$
    1999/07/27 Creation.
BUGS
    - It's not implemented yet.
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#ifndef __O2CI8ServiceContext_h__
#define __O2CI8ServiceContext_h__
#include <O2CI8Object.h>
#include <O2CI8ServerConnection.h>
#include <O2CI8UserSession.h>
#include <O2CI8Transaction.h>
#include <O2CI8Statement.h>


@interface O2CI8ServiceContext:O2CI8Object
{
	O2CI8ServerConnection*   serverConnection;
	O2CI8UserSession*        userSession;
	O2CI8Transaction*        transaction;
	BOOL                    sessionIsActive;
}

// NSObject methods:

    -(id)init;
		/*
			RETURN:     self.
			NOTE:       when no error occured, [self handleIsValid].
		*/

    -(void)dealloc;


// O2CI8Object methods:

	// no override.


    
// O2CI8ServiceContext methods:

	-(BOOL)beginSessionWithServer:(O2CI8ServerConnection*)server
			forUser:(O2CI8UserSession*)user withAuthentificationModes:(ub4)mode;
		/*
			PRE:        ![self sessionIsActive].

			DO:         Authenticate a user against the server set in the 
						service context.

			POST:       result ==> [self sessionIsActive].

			RAISE:      O2CI8Object_eInvalidHandle.
			RAISE:      O2CI8Object_eInvalidState.

			RETURN:     Authentification was sucessfull.

			mode:       OCI_DEFAULT - in this mode, the user session context 
                                      returned may only ever be set with the 
									  same server context specified in svchp. 

						OCI_MIGRATE - in this mode, the new user
									  session   context may be set in
									  a service handle with a
									  different server handle. This
									  mode  establishes the user
									  session context.  To create a
									  migratable session, the service
									  handle must already be set with
									  a non-migratable user session. A
									  migratable  session must have a
									  non-migratable parent  session.

						OCI_SYSDBA - in this mode, the user is 
									 authenticated for SYSDBA access. 

						OCI_SYSOPER - in this mode, the user is 
									  authenticated for SYSOPER access. 

						OCI_PRELIM_AUTH - this mode may only be used with
										  OCI_SYSDBA or OCI_SYSOPER to  
										  authenticate for certain 
										  administration tasks. 
		*/


	-(void)endSession;
		/*
			PRE:        [self sessionIsActive].

			DO:         De-authenticate the user.

			POST:       ![self sessionIsActive].

			RAISE:      O2CI8Object_eInvalidHandle.
			RAISE:      O2CI8Object_eInvalidState.
		*/


	-(BOOL)sessionIsActive;
		/*
			RETURN:     whether the session is active.
		*/

	-(O2CI8ServerConnection*)serverConnection;
		/*
			PRE:        [self handleIsValid].
			RAISE:      O2CI8Object_eInvalidHandle when ![self handleIsValid].
			RETURN:     the server connection.
		*/

	-(O2CI8UserSession*)userSession;
		/*
			PRE:        [self handleIsValid].
			RAISE:      O2CI8Object_eInvalidHandle when ![self handleIsValid].
			RETURN:     the user session.
		*/

	-(O2CI8Transaction*)transaction;
		/*
			PRE:        [self handleIsValid].
			RAISE:      O2CI8Object_eInvalidHandle when ![self handleIsValid].
			RETURN:     the transaction.
		*/


// Transaction management methods:

	-(BOOL)transactionCommit:(ub4)flags;
		/*
			DO:         commit the current transaction.

			RETURN:     whether the transaction could be correctly commited.
						(otherwise, it may have been rolled back, or may
						be in doubt).

			flags:      is used for one-phase commit optimization in global 
						transactions. If the transaction is non-distributed, 
						the flags parameter is ignored, and OCI_DEFAULT can 
						be passed as its value. OCI applications managing 
						global transactions should pass a value of 
						OCI_TRANS_TWOPHASE to the flags parameter for a 
						two-phase commit. The default is one-phase commit.
		*/


	-(BOOL)transactionRollback;
		/*
			DO:         roll back the current transaction.

			RETURN:     whether the transaction could be correctly rolled back.
		*/


	-(BOOL)transactionStartBranchTimeout:(uword)timeout flags:(ub4)flags;
		/*
			DO:         starts a new transaction branch.

			NOTE:       transactionCommit: and transactionRollback automatically
						start a new transaction.

			timeout:    The time, in seconds, to wait for a transaction to 
						become available for resumption when
						OCI_TRANS_RESUME is specified. When
						OCI_TRANS_NEW is specified, the timeout
						parameter indicates the number of seconds the
						transaction can be inactive before it is
						automatically aborted by the system. A
						transaction is inactive between the time it is
						detached (with OCITransDetach()) and the time
						it is resumed with OCITransStart().

			flags:      Specifies whether a new transaction is being 
						started or an existing transaction is being
						resumed. Also specifies serializiability or
						read-only status. More than a single value can
						be specified. By default, a read/write
						transaction is started. The flag values are:
						OCI_TRANS_NEW - starts a new transaction branch. 
										By default starts a tightly coupled 
										and migratable branch. 
						OCI_TRANS_TIGHT - explicitly specifies a tightly coupled 
										branch.
						OCI_TRANS_LOOSE - specifies a loosely coupled branch 
						OCI_TRANS_RESUME - resumes an existing transaction branch. 
						OCI_TRANS_READONLY - start a read-only transaction 
						OCI_TRANS_SERIALIZABLE - start a serializable transaction. 
		*/


	-(BOOL)transactionDetach;
		/*
			DO:         Detaches the current transaction.

						Detaches a global transaction from the service
						context handle. The transaction currently
						attached to the service context handle becomes
						inactive at the end of this call. The
						transaction may be resumed later by calling
						OCITransStart(), specifying a flags value of
						OCI_TRANS_RESUME.

						When a transaction is detached, the value
						which was specified in the timeout parameter
						of OCITransStart() when the transaction was
						started is used to determine the amount of
						time the branch can remain inactive before
						being deleted by the server's PMON process.  

			RETURN:     whether the transaction has been successfully detached.
		*/


	-(BOOL)transactionPrepare;
		/*
			DO:         Prepares the specified global transaction for commit. 
						This call is valid only for global transactions. 

			RETURN:     whether the transaction has been successfully prepared.
					    The call returns OCI_SUCCESS_WITH_INFO if the 
						transaction has not made any changes. The error 
						handle will indicate that the transaction is read-only. 
		*/

	-(BOOL)transactionForget;
		/*
			DO:         Forgets a heuristically completed global transaction. 
						The server deletes the status of the transaction from 
						the system's pending transaction table. 
						The XID of the transaction to be forgotten is set as 
						an attribute of the transaction handle (OCI_ATTR_XID). 

			RETURN:     whether the transaction has been successfully forgotten.
		*/


// Statement execution methods:

	-(BOOL)executeStatement:(O2CI8Statement*)statement
			iterations:(ub4)nIter rowOffset:(ub4)rOffset 
			snapIn:(const OCISnapshot*)sIn snapOut:(OCISnapshot*)sOut
			mode:(ub4)mode;
		/*
			PRE:        the statement has been prepared.
			DO:         execute the statement.
			RETURN:     whether the statement has been executed successfully.
		*/


@end //O2CI8ServiceContext;

#endif /*__O2CI8ServiceContext_h__*/

/*** O2CI8ServiceContext.h -- 1999/07/27 06:35:55 -- <PJB> ***/
