/******************************************************************************
FILE:               Oracle8Adaptor.m
LANGUAGE:           Objective-C
SYSTEM:             GNUstep
USER-INTERFACE:     None
DESCRIPTION

    This class is the GNUstep/db adaptor to the Oracle8 databases.
	Its implementation is based on the OCI v8, and should be compatible
	with, both Oracle87, Oracle88 and future Oracle89 databases.

AUTHORS
    <PJB> Pascal J. Bourguignon <pjb@imaginet.fr>
MODIFICATIONS
    1999/07/24 Creation.
BUGS
    - The name of this adaptor should probably be more specific.

LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999
	Copyright Software Builders     1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#include <Oracle8Adaptor.h>
#include <Oracle8Context.h>
#include <Oracle8Context-ForFriends.h>
#include <Oracle8Channel.h>
#include <Oracle8Exceptions.h>
#include <Oracle8SQLExpression.h>
#include <Oracle8Values.h>
#include <Oracle8Utilities.h>

#include <O2CI8Environment.h>
#include <O2CI8ServerConnection.h>
#include <O2CI8UserSession.h>

// connectionDictionary keys:
NSString* Oracle8Adaptor_cdUsername=@"userName";
NSString* Oracle8Adaptor_cdPassword=@"password";
NSString* Oracle8Adaptor_cdConnectionString=@"connectionString";
NSString* Oracle8Adaptor_cdServerId=@"serverId";
NSString* Oracle8Adaptor_cdHostMachine=@"hostMachine";
NSString* Oracle8Adaptor_cdNLS_LANG=@"NLS_LANG";
// Check normalizedConnectionDictionary: if you add keys here.


// exceptions:
DECLARE_SYMBOL(Oracle8Adaptor_eBadInitializer)
DECLARE_SYMBOL(Oracle8Adaptor_eInternalError)


// -------------------------------------------------------------------------- //

@interface Oracle8Adaptor(Private)


	-(BOOL)oracleTypeIsStringType:(NSString*)externalType;
		/*
			RETURN:     Whether externalType is a string type, ie. one of:
							- char
							- char(n)
							- long
							- long(n)
							- varchar2
							- varchar2(n)
		*/

	-(NSString*)quotedStringValue:(NSString*)value;
		/*
			RETURN:     A quoted version of the string value. Every single 
						quote in value is doubled, and the whole string is 
						single quoted.

			NOTE:       New-lines are not quoted, since they're well 
						accepted by	Oracle8 on inserts.
		*/


	-(NSString*)dateValue:(NSDate*)value;
		/*
			RETURN:     A SQL expression of the date value.

			NOTE:       The returned string is in the form: 
							TO_DATE('date value','date format')
		*/


	-(unsigned int)countCaseInsensitiveString:(NSString*)string
				  isPresentInArray:(NSArray*)array;
        /*
		    RETURN:     The number of occurence of string (case insensitively)
                        that are found in array.
        */

    -(NSDictionary*)normalizedConnectionDictionary:(NSDictionary*)connDict;
        /*
            RETURN:     A dictionary with the keys normalized, that is, 
                        capitalized as defined in the Oracle8Adaptor_cd* strings.
        */

@end // Oracle8Adaptor(Private).


// -------------------------------------------------------------------------- //

@implementation Oracle8Adaptor


// NSObject methods:

	-(id)init
	{
		return([self initWithName:@"Oracle8"]);
	}//init;


	-(void)dealloc
	{

		[super dealloc];
	}//dealloc;


// EOAdaptor methods:

	-(id)initWithName:(NSString*)aName
	{
		self=[super initWithName:aName];
		if(self!=nil){
            validatingConnectionDictionary=NO;
			[O2CI8Environment initializeWithMode:OCI_THREADED];
		}
		return(self);
	}//initWithName:;



    -(void)setConnectionDictionary:(NSDictionary*)aDictionary
    {
        [super setConnectionDictionary:
                   [self normalizedConnectionDictionary:aDictionary]];
    }//setConnectionDictionary:;


	-(BOOL)hasValidConnectionDictionary
	{
        /*
            Well, we should not need to handle case insensivity here since
            the connection dictionary has been normalized in 
            setConnectionDictionary:, but this came later, and anyway, 
            a subclass could un-normalize the connection dictionary.
            (That's not a time critical method).
        */
		NSArray*          keys;
		EOAdaptorContext* context;
		EOAdaptorChannel* channel;

		// The connectionDictionary must exist.
		if(nil==connectionDictionary){
			return(NO);
		}
		
		keys=[connectionDictionary allKeys];
		
		// Both the username and the password must be present.
		if(1!=[self countCaseInsensitiveString:Oracle8Adaptor_cdUsername
				 isPresentInArray:keys]){
			return(NO);
		}
		if(1!=[self countCaseInsensitiveString:Oracle8Adaptor_cdPassword
				 isPresentInArray:keys]){
			return(NO);
		}
		switch([self countCaseInsensitiveString:Oracle8Adaptor_cdConnectionString
				 isPresentInArray:keys]){
		case 0:
			// If there is no connectionString, will need to have
			// both the serverId and the hostMachine exactly once.
			if(1!=[self countCaseInsensitiveString:Oracle8Adaptor_cdServerId
						isPresentInArray:keys]){
				return(NO);
			}
			if(1!=[self countCaseInsensitiveString:Oracle8Adaptor_cdHostMachine
						isPresentInArray:keys]){
				return(NO);
			}
			break;
		case 1:
			// Then the others may be present or not, 
			// but we don't want two of them.
			if(2<=[self countCaseInsensitiveString:Oracle8Adaptor_cdServerId
						isPresentInArray:keys]){
				return(NO);
			}
			if(2<=[self countCaseInsensitiveString:Oracle8Adaptor_cdHostMachine
						isPresentInArray:keys]){
				return(NO);
			}
			break;
		defaut:// too many.
			return(NO);
		}
		// All the remainder is optional and not needed to make a valid 
		// connection dictionary. However, we check if exatly 0 or 1 exists.
		if(2<=[self countCaseInsensitiveString:Oracle8Adaptor_cdNLS_LANG
				 isPresentInArray:keys]){
			return(NO);
		}
        /*
            Why this `validatingConnectionDictionary' flag?
            Because this method will be called by the channel in openChannel,
            and this could provoke an infinite recursion. So there...
        */
        if(validatingConnectionDictionary){
            return(YES);
        }
		// Ok, all is well. Let's try to connect.
        validatingConnectionDictionary=YES;
        NS_DURING
            context=[self createAdaptorContext];
		    if(context==nil){
                validatingConnectionDictionary=NO;
                NS_VALRETURN(NO);
            }
            channel=[context createAdaptorChannel];
            if(channel==nil){
                validatingConnectionDictionary=NO;
                NS_VALRETURN(NO);
            }
            if([channel openChannel]){
                [channel closeChannel];
                validatingConnectionDictionary=NO;
                NS_VALRETURN(YES);
            }else{
                validatingConnectionDictionary=NO;
                NS_VALRETURN(NO);
            }
        NS_HANDLER
            validatingConnectionDictionary=NO;
            [localException raise];
        NS_ENDHANDLER
	}//hasValidConnectionDictionary;


	-(EOAdaptorContext*)createAdaptorContext
	{
		return([super createAdaptorContext]);
	}//createAdaptorContext;


	-(Class)expressionClass
	{
		return([Oracle8SQLExpression class]);
	}//expressionClass;


	-(Class)adaptorContextClass
	{
		return([Oracle8Context class]);
	}//adaptorContextClass;


	-(Class)adaptorChannelClass
	{
		return([Oracle8Channel class]);
	}//adaptorChannelClass;


	-(BOOL)isValidQualifierType:(NSString*)aTypeName
	{
		NOT_IMPLEMENTED_YET;
		NSLog(@"-[Oracle8Adapter isValidQualifierType:@\"%@\"]?\n",aTypeName);
		return(YES);
	}//isValidQualifierType:;


	-(id)formatAttribute:(EOAttribute*)attribute
	{
		// super does what is needed (returns column name).
		return([super formatAttribute:attribute]);
	}//formatAttribute:;



	-(id)formatValue:(id)value forAttribute:(EOAttribute*)attribute
	{
		// SEE: This implementation is insufficient.
		if([self oracleTypeIsStringType:[attribute externalType]]){
			return([self quotedStringValue:value]);
		}else if([value isKindOfClass:[NSDate class]]){
			return([self dateValue:value]);
		}else{
			return([super formatValue:value forAttribute:attribute]);
		}
	}//formatValue:;



// Oracle8Adaptor methods:

	// None. What would be the purpose of an EO adaptor layer if each and
	// every one EOAdaptor subclass  would add it's own specific methods?
	// Let the client use only the EOAdaptor API to access "portably" our
	// database.

@end // Oracle8Adaptor;


// -------------------------------------------------------------------------- //

@implementation Oracle8Adaptor(Private)


	-(BOOL)oracleTypeIsStringType:(NSString*)externalType
	{
		NSRange   firstWordRange;
		NSRange   afterFirstWordRange;
		static NSCharacterSet* spaceOrParen=nil;
		if(spaceOrParen==nil){
			spaceOrParen=[[NSCharacterSet characterSetWithCharactersInString:@" ("]
							retain];
		}
		firstWordRange.location=0;
		afterFirstWordRange=[externalType rangeOfCharacterFromSet:spaceOrParen];
		if(afterFirstWordRange.length==0){
			firstWordRange.length=[externalType length];
		}else{
			firstWordRange.length=afterFirstWordRange.location;
		}
		return((NSOrderedSame==[externalType compare:@"char"
					options:NSCaseInsensitiveSearch	range:firstWordRange])
		     ||(NSOrderedSame==[externalType compare:@"long"
					options:NSCaseInsensitiveSearch	range:firstWordRange])
		     ||(NSOrderedSame==[externalType compare:@"varchar2"
					options:NSCaseInsensitiveSearch	range:firstWordRange]));
	}//oracleTypeIsStringType:;


	-(NSString*)quotedStringValue:(NSString*)value
	{
		NSString*       result;
		unsigned int    blen=0;
		unsigned int    vind=0;
		unsigned int    vlen=[value length];
		unichar*        buffer=(unichar*)malloc(sizeof(unichar)*2*(1+vlen));
		buffer[blen++]=(unichar)'\'';
		while(vind<vlen){
			unichar current=[value characterAtIndex:vind];
			if(current==(unichar)'\''){
				buffer[blen++]=(unichar)'\'';
				buffer[blen++]=(unichar)'\'';
			}else{
				buffer[blen++]=current;				
			}
			vind++;
		}
		buffer[blen++]=(unichar)'\'';
		result=[[NSString alloc]initWithCharacters:buffer length:blen];
		free(buffer);
		return([result autorelease]);
	}//quotedStringValue:;


	-(NSString*)dateValue:(NSDate*)value
	{
		static NSTimeZone* GMT=nil;
		if(GMT==nil){
			GMT=[NSTimeZone timeZoneForSecondsFromGMT:0];
		}
		return([NSString stringWithFormat:
				@"TO_DATE(%@,'SYYYY/MM/DD HH24:MI/SS')",
				[self quotedStringValue:
					[value descriptionWithCalendarFormat:@"%Y/%m/%d %H:%M:%S"
							timeZone:GMT locale:nil]]]);
	}//quotedDateValue:;


	-(unsigned int)countCaseInsensitiveString:(NSString*)string
				  isPresentInArray:(NSArray*)array
	{
		unsigned int count=[array count];
		unsigned int sc=0;
		unsigned int i=0;
		while(i<count){
			if([string compare:[array objectAtIndex:i] 
					   options:NSCaseInsensitiveSearch]==NSOrderedSame){
				sc++;
			}
			i++;
		}
		return(sc==1);
	}//countCaseInsensitiveString:isPresentInArray:;



    -(NSDictionary*)normalizedConnectionDictionary:(NSDictionary*)connDict
    {
        static NSString*        normalizedKeys[7]={nil};
		NSArray*                keys;
        unsigned int            index;
        NSEnumerator*           enumerator;
        NSString*               current;
        NSMutableDictionary*    normalized=[NSMutableDictionary 
                                    dictionaryWithCapacity:[connDict count]];

        if(normalizedKeys[0]==nil){
            unsigned int i=1;
            normalizedKeys[i++]=Oracle8Adaptor_cdUsername;
            normalizedKeys[i++]=Oracle8Adaptor_cdPassword;
            normalizedKeys[i++]=Oracle8Adaptor_cdConnectionString;
            normalizedKeys[i++]=Oracle8Adaptor_cdServerId;
            normalizedKeys[i++]=Oracle8Adaptor_cdHostMachine;
            // Don't forget to update the array size if you add a key here!
            normalizedKeys[i++]=nil;
            normalizedKeys[0]=Oracle8Adaptor_cdNLS_LANG;
            // (That's a little kludge to be thread-clean here).
        }

		if(nil==connDict){
			return(nil);
		}
		
		keys=[connDict allKeys];
        enumerator=[keys objectEnumerator];
        while(nil!=(current=[enumerator nextObject])){
            index=0;    
            while(normalizedKeys[index]!=nil){
                if([current compare:normalizedKeys[index] 
					   options:NSCaseInsensitiveSearch]==NSOrderedSame){
                    [normalized setObject:[connDict objectForKey:current]
                                forKey:normalizedKeys[index]];
                    break;
                }
                index++;
            }
            if(normalizedKeys[index]==nil){
                [normalized setObject:[connDict objectForKey:current] 
                            forKey:current];
                NSLog(NSLocalizedStringFromTableInBundle(@"Warning! The key \"%@\" in the connection dictionary is not recognized by the Oracle8Adaptor.\n",@"Oracle8Adaptor",Oracle8Utilities_Bundle(),@"Message given when an unknown key is included in the connection dictionary. [1]:The unknown key."),current);
            }
        }
        return(normalized);
    }//normalizedConnectionDictionary:;


@end // Oracle8Adaptor(Private);

// -------------------------------------------------------------------------- //

/*** Oracle8Adaptor.m -- 1999/07/24 04:44:48 -- PJB ***/
