/******************************************************************************
FILE:               Oracle8Buffer.m
LANGUAGE:           Objective-C
SYSTEM:             None
USER-INTERFACE:     None
DESCRIPTION
    $Header$
    This is the implementation of the Oracle8Buffer class.

    The class Oracle8Buffer encapsulate an O2CI8Define and the buffer data
	passed to it. It's able to convert data obtained from a fetch into
	OpenStep instances of NSNumber, NSString, NSDate, NSData.

    The implementation of the class Oracle8Buffer xxxxxxx.

AUTHORS
    <PJB>  Pascal J. Bourguignon  <pjb@imaginet.fr>
MODIFICATIONS
    $Log$
    1999/08/02 Creation.
BUGS
    - ...
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#include <Oracle8Buffer.h>
#include <Oracle8Adaptor.h>
#include <Oracle8Utilities.h>
#include <O2CI8Column.h>
#include <O2CI8Define.h>
#include <O2CI8Statement.h>
#include <oci.h>
#include <NSString-Oracle8Adaptor_LongLong.h>


#define INDICATOR_MAGIC (-5431) // anything less than -3 should do.


@interface Oracle8Buffer(Private)

	-(void)releaseData;
		/*
			POST:       self is in mint condition.
		*/

	-(void)getDataType:(ub2*)dt andDataLength:(ub2*)dl 
			forAttribute:(EOAttribute*)attribute
			andColumn:(O2CI8Column*)column;
		/*
			DO:         Determine the dataType and dataLength for the buffer.
						This is done by choosing a dataType adapted to the 
						attribute (in order to let the conversions be done by
						OCI), or to the column in the last resort.

			POST:       dt and dl are set apropriately for the expected type
						for the attribute, or by default, for the column.
		*/



	-(BOOL)isNullOrWhat;
		/*
			PRE:        A fetch has been done ([statement fetchRow:1]).

			DO:         Checks the indicator.

			RAISE:      Oracle8Adaptor_eInternalError when the data has been 
														  truncated.

			RETURN:     column value is null.
		*/


@end //Oracle8Buffer(Private)


// -------------------------------------------------------------------------- //

@implementation Oracle8Buffer(Private)

	-(void)releaseData
	{
		[attribute release]; attribute=nil;
        [column    release]; column=nil;
		[define    release]; define=nil;
		if(buffer!=nil){
			free(buffer); buffer=nil;
		}
	}//releaseData;

	-(void)getDataType:(ub2*)dt andDataLength:(ub2*)dl 
			forAttribute:(EOAttribute*)anAttribute
			andColumn:(O2CI8Column*)aColumn
	{
		// http://technet.oracle.com/doc/server.805/a58234/datatype.htm#431514
		static struct { NSString* cn; NSString* st; ub2 dt; ub2 dl; } 
		mapping[]={
			{@"NSNumber",      @"d",SQLT_FLT,  sizeof(double)},
			{@"NSNumber",      @"i",SQLT_INT,  sizeof(long int)},
			{@"NSNumber",      @"s",SQLT_INT,  sizeof(short int)},
			{@"NSNumber",      @"c",SQLT_INT,  sizeof(char)},
			{@"NSNumber",      nil, SQLT_AVC,  30},
			{@"NSString",      nil, SQLT_AVC,  4001},
			{@"NSCalendarDate",nil, SQLT_DAT,  7},
			{@"NSDate",        nil, SQLT_DAT,  7},
			{@"NSData",        nil, SQLT_BIN,  2000},
			{nil,              nil, 0,         0}};
		int i=0;
		NSString* vcn=[anAttribute valueClassName];
		NSString* vt=[anAttribute valueType];
		while(!((mapping[i].cn==nil)
			||(([vcn isEqualToString:mapping[i].cn])
				&&((mapping[i].st==nil)
					||([vt isEqualToString:mapping[i].st]))))){
			i++;
		}
		if(mapping[i].cn==nil){
			(*dt)=[aColumn dataType];
			(*dl)=[aColumn dataSize];
		}else{
			(*dt)=mapping[i].dt;
			(*dl)=mapping[i].dl;
		}
	}//getDataType:andDataLength:forAttribute:andColumn:;





	-(BOOL)isNullOrWhat
	{
		switch(indicator){
		case -2:
			[NSException raise:Oracle8Adaptor_eInternalError
				format:NSLocalizedStringFromTableInBundle(@"Truncated data for attribute %@, column=%@, allocated length=%d, value length is greater than 32767",@"Oracle8Adapter",Oracle8Utilities_Bundle(),@"Exception raison when some big data has been truncated upon a fetch. [1]:attribute name. [2]:column name. [3]:allocated length (that was too small)."),
						[attribute name],[attribute columnName],bufferSize];
			
		case -1:
			return(YES);

		case 0:
			return(NO);
		
		case INDICATOR_MAGIC:
			[NSException raise:Oracle8Adaptor_eInternalError
					format:NSLocalizedStringFromTableInBundle(@"Did you do a fetch? You should have, before trying to get data from the buffer!",@"Oracle8Adapter",Oracle8Utilities_Bundle(),@"Exception raison given when the client tries to get data from the buffer before fetching a row.")];
			
		default:
			if(indicator>0){
				[NSException raise:Oracle8Adaptor_eInternalError
					format:NSLocalizedStringFromTableInBundle(@"Truncated data for attribute %@, column=%@, allocated length=%d, value length=%d",@"Oracle8Adapter",Oracle8Utilities_Bundle(),@"Exception raison when some big data has been truncated upon a fetch. [1]:attribute name. [2]:column name. [3]:allocated length (that was too small). [4]:actuall data length."),
						[attribute name],[attribute columnName],
						bufferSize,indicator];
			}else{
				[NSException raise:Oracle8Adaptor_eInternalError
					format:NSLocalizedStringFromTableInBundle(@"Unexpected indicator value (ind=%d<-2)",@"Oracle8Adapter",Oracle8Utilities_Bundle(),@"Exception raison when an unexpected (and undocumented) indicator code is returned. [1]:the invalid indicator value"),indicator];
			}
		}
	}//isNullOrWhat;


@end //Oracle8Buffer(Private)

// -------------------------------------------------------------------------- //


@implementation Oracle8Buffer

    static const char* rcsid="$Id";

// NSObject methods:

    -(id)init
    {
        self=[super init];
        if(self!=nil){
			column=nil;
			define=nil;
			buffer=nil;
			bufferSize=0;
			dataType=0;
			indicator=INDICATOR_MAGIC;
			dataLength=0;
			returnCode=0;
			attribute=nil;
        }
        return(self);
    }//init;


    -(void)dealloc
    {
		[self releaseData];
        [super dealloc];
    }//dealloc;


// NSObject methods:


    
// Oracle8Buffer methods:


	-(BOOL)defineForColumn:(O2CI8Column*)aColumn 
			atPosition:(ub4)position
			inStatement:(O2CI8Statement*)aStatement
			asAttribute:(EOAttribute*)anAttribute
	{
		ub2 dl;
		if(column!=nil){
			[self releaseData];
		}
		column=[aColumn retain];
		attribute=[anAttribute retain];
		bufferSize=[column dataSize];
		buffer=(void*)malloc(bufferSize);
		[self getDataType:&dataType andDataLength:&dl
				forAttribute:attribute andColumn:column];
		bufferSize=(ub4)dl;				
		indicator=INDICATOR_MAGIC;
		dataLength=0;
		returnCode=0;
		define=[aStatement defineAtPosition:position
			buffer:buffer bufferSize:bufferSize dataType:dataType
			indicator:&indicator dataLength:&dataLength returnCode:&returnCode];
		if(define==nil){
			[self releaseData];
			return(NO);
		}else{
			return(YES);
		}
	}//defineForColumn:atPosition:inStatement:;


	-(NSData*)asData
	{
		if([self isNullOrWhat]){
			return([EONull null]);
		}else{
			return([NSData dataWithBytes:buffer length:dataLength]);
		}
	}//asData;


	-(NSNumber*)asNumber
	{
		if([self isNullOrWhat]){
			return([EONull null]);
		}else{
			switch(dataType){
			case SQLT_INT:
				switch(dataLength){
				case 1:
					return([NSNumber numberWithChar:*(char*)buffer]);
				case 2:
					return([NSNumber numberWithShort:*(short*)buffer]);
				case 4:
					return([NSNumber numberWithLong:*(long*)buffer]);
				default:
				}					
				break;
			case SQLT_FLT:
				switch(dataLength){
				case 4:
					return([NSNumber numberWithFloat:*(float*)buffer]);
				case 8:
					return([NSNumber numberWithDouble:*(double*)buffer]);
				default:
				}
				break;
			case SQLT_AVC:
				return([NSNumber numberWithLongLong:[
					 [NSString stringWithCString:(char*)buffer]
							oracle8AdaptorLongLongValue]]);
			default:
				break;
			}
		}
		[NSException raise:Oracle8Adaptor_eInternalError
			format:NSLocalizedStringFromTableInBundle(@"Unexpected dataType/dataSize for a NSNumber. type=%d  size=%d",@"Oracle8Adapter",Oracle8Utilities_Bundle(),@"Exception raison given when data size for a number is not handled. [1]:the data type. [2]:the data size."),dataType,dataLength];
	}//asNumber;


	-(NSString*)asString
	{
		if([self isNullOrWhat]){
			return([EONull null]);
		}else{
			return([NSString stringWithCString:buffer]);
		}
	}//asString;


	-(NSDate*)asDate
	{
		return([self asCalendarDate]);
	}//asDate;


	-(NSCalendarDate*)asCalendarDate
	{
		if([self isNullOrWhat]){
			return([EONull null]);
		}else{
			static NSTimeZone* GMT=nil;
			signed char* db=(signed char*)buffer;
			if(GMT==nil){
				GMT=[NSTimeZone timeZoneWithName:@"GMT"];
			}
// NSLog(@"date: %3d %3d / %2d / %2d %2d : %2d : %2d\n",
// db[0],db[1],db[2],db[3],db[4],db[5],db[6]);
			return([NSCalendarDate dateWithYear:
				(((int)((unsigned char)db[0]))-100)*100
				+(((int)((unsigned char)(db[1])))-100)
				month:db[2] day:db[3] 
				hour:db[4]-1 minute:db[5]-1 second:db[6]-1 timeZone:GMT]);
		}
	}//asCalendarDate;



	-(void)fillRow:(NSMutableDictionary*)row
	{
		static struct { NSString* cn; SEL s; } mapping[]={
			{@"NSNumber",      @selector(asNumber)},
			{@"NSString",      @selector(asString)},
			{@"NSCalendarDate",@selector(asCalendarDate)},
			{@"NSDate",        @selector(asDate)},
			{@"NSData",        @selector(asData)},
			{nil,              @selector(asData)}};

		NSString* vcn=[attribute valueClassName];
		int i;
		i=0;
		while((mapping[i].cn!=nil)&&(![vcn isEqualToString:mapping[i].cn])){
			i++;
		}
		[row setObject:[self performSelector:mapping[i].s]
				forKey:[attribute name]];
//NSLog(@"fillRow: key=%-20@ object=%@\n",[attribute name],[[row objectForKey:[attribute name]]description]);
	}//fillRow:;


@end //Oracle8Buffer;

/*** Oracle8Buffer.m -- 1999/08/02 20:31:47 -- <PJB> ***/

