/******************************************************************************
FILE:               Oracle8Channel.m
LANGUAGE:           Objective-C
SYSTEM:             GNUstep
USER-INTERFACE:     None
DESCRIPTION

    This class is the GNUstep/db channel to the Oracle8 databases.
	Its implementation is based on the OCI v8, and should be compatible
	with, both Oracle87, Oracle88 and future Oracle89 databases.


AUTHORS
    <PJB> Pascal J. Bourguignon <pjb@imaginet.fr>
MODIFICATIONS
    1999/07/24 Creation.
BUGS
    - ...
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999
	Copyright Software Builders     1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#include <Oracle8Channel.h>
#include <Oracle8Adaptor.h>
#include <Oracle8Context.h>
#include <Oracle8Context-ForFriends.h>
#include <Oracle8Buffer.h>
#include <Oracle8Exceptions.h>
#include <Oracle8Values.h>
#include <Oracle8Utilities.h>
#include <oci.h>
#include <O2CI8ServiceContext.h>
#include <O2CI8Statement.h>
#include <O2CI8Parameter.h>
#include <O2CI8Column.h>
#include <NSArray-Oracle8Adaptor_Perform.h>

//exceptions:
DECLARE_SYMBOL(Oracle8Channel_eAttributeNotInResultSet)


// -------------------------------------------------------------------------- //

@interface Oracle8Channel(Private)

	-(void)setAttribute:(EOAttribute*)attribute withDataType:(ub2)dataType;

	-(void)makeFetchResults;
		/*
			POST:       fetchResults contains O2CI8Column instances in its even
						slots and corresponding EOAttribute instances in its
						odd slots.
		*/

	-(NSMutableArray*)defineBuffersForAttributes:(NSArray*)attributes;
		/*
			PRE:        fetchResults!=nil.

			SEE:        -makeFetchResults.

			RETURN:     an autoreleased array of Oracle8Buffer instances that 
                        has been defined for the statement.
		*/


@end //Oracle8Channel(Private);


// -------------------------------------------------------------------------- //

@implementation Oracle8Channel(Private)

	-(void)setAttribute:(EOAttribute*)attribute withDataType:(ub2)dataType
	{
		switch(dataType){
		case SQLT_AFC:
			// char[n] 
			[attribute setExternalType:@"CHAR"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_AVC:
			// char[n+1] 
			[attribute setExternalType:@"CHARZ"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_BIN:
			// unsigned char[n] 
			[attribute setExternalType:@"RAW"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_BLOB:
			// OCILobLocator (see note 3) 
			[attribute setExternalType:@"Binary LOB"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_CHR:
			// char[n] 
			[attribute setExternalType:@"VARCHAR2"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_CLOB:
			// OCILobLocator (see note 3) 
			[attribute setExternalType:@"Character LOB"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_DAT:
			// char[7] 
			[attribute setExternalType:@"DATE"];
			[attribute setValueClassName:@"NSDate"];
			break;
		case SQLT_FILE:
			// OCILobLocator 
			[attribute setExternalType:@"FILE"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_FLT:
			// float, double 
			[attribute setExternalType:@"FLOAT"];
			[attribute setValueClassName:@"NSNumber"];
			[attribute setValueType:@"d"];
			break;
		case SQLT_INT:
			// signed short, signed int 
			// signed int, signed long 
			// signed char 
			[attribute setExternalType:@"INTEGER"];
			[attribute setValueClassName:@"NSNumber"];
			[attribute setValueType:@"i"];
			break;
		case SQLT_LAB:
			// char[n] 
			[attribute setExternalType:@"MLSLABEL"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_LBI:
			// unsigned char[n] 
			[attribute setExternalType:@"LONG RAW"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_LNG:
			// char[n] 
			[attribute setExternalType:@"LONG"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_LVB:
			// unsigned char[n+sizeof(integer)] 
			[attribute setExternalType:@"LONG VARRAW"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_LVC:
			// char[n+sizeof(integer)] 
			[attribute setExternalType:@"LONG VARCHAR"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_NTY:
			// struct 
			[attribute setExternalType:@"NAMED DATA TYPE"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_NUM:
			// unsigned char[21] 
			[attribute setExternalType:@"NUMBER"];
			[attribute setValueClassName:@"NSNumber"];
		    [attribute setValueType:@"d"];
		break;
    /*
		case SQLT_ODT:
			// OCIDate (see note 2)
			[attribute setExternalType:@"OCI date type"];
			break;
	*/
		case SQLT_RDD:
			// OCIRowid 
			[attribute setExternalType:@"ROWID"];
			[attribute setValueClassName:@"NSData"];
			break;
	/*
		case SQLT_REF:
			// OCIRef 
			[attribute setExternalType:@"REF"];
			break;
	*/
		case SQLT_RID:
			// char[n] (see note 1)
			[attribute setExternalType:@"ROWID"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_STR:
			// char[n+1] 
			[attribute setExternalType:@"STRING"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_UIN:
			// unsigned 
			[attribute setExternalType:@"UNSIGNED INT"];
			[attribute setValueClassName:@"NSNumber"];
			[attribute setValueType:@"i"];
			break;
		case SQLT_VBI:
			// unsigned char[n+sizeof(short integer)] 
			[attribute setExternalType:@"VARRAW"];
			[attribute setValueClassName:@"NSData"];
			break;
		case SQLT_VCS:
			// char[n+sizeof(short integer)] 
			[attribute setExternalType:@"VARCHAR"];
			[attribute setValueClassName:@"NSString"];
			break;
		case SQLT_VNU:
			// char[22] 
			[attribute setExternalType:@"VARNUM"];
			[attribute setValueClassName:@"NSString"];
			break;
	/*
		case SQLT_VST:
			// OCIString (see note 2)
			[attribute setExternalType:@"OCI string type"];
			break;
	*/
		default:
			NOT_IMPLEMENTED_YET; // SEE: Raise an exception.
			break;
		}
	}//setAttribute:withDataType:;


	-(void)makeFetchResults
	{
		NSArray*        parameters=[statement getParameters];
		NSEnumerator*   enumerator=[parameters objectEnumerator];
		O2CI8Parameter*  parameter;
		int             index=0;
		[fetchResults release];
		fetchResults=[[NSMutableArray alloc]initWithCapacity:2*[parameters count]];
		while(parameter=[enumerator nextObject]){
			EOAttribute* attribute=[[[EOAttribute alloc] initWithName:
				[NSString stringWithFormat:@"Attribute%d",index]]autorelease];
			if([parameter isKindOfClass:[O2CI8Column class]]){
				O2CI8Column* column=(O2CI8Column*)parameter;
				[attribute setColumnName:[column name]];
				[attribute setExternalType:[column typeName]];
				[self setAttribute:attribute withDataType:[column dataType]];
				[fetchResults addObject:column];
				[fetchResults addObject:attribute];
			}else{
				[fetchResults release];
				fetchResults=nil;
				[NSException raise:Oracle8Adaptor_eInternalError
					format:NSLocalizedStringFromTableInBundle(@"A fetch parameter is not a column!",@"Oracle8Channel",Oracle8Utilities_Bundle(),@"Exception raison given when a statement parameter is not a column.")];
			}
			index++;
		}
	}//makeFetchResults;


	-(NSArray*)defineBuffersForAttributes:(NSArray*)attributes
	{
		unsigned int fcount=[fetchResults count];
		unsigned int count=fcount/2;
		NSMutableArray* buffers=[NSMutableArray arrayWithCapacity:count];
		NSEnumerator*   attrEnum=[attributes objectEnumerator];
		EOAttribute*    attribute;

		while(nil!=(attribute=[attrEnum nextObject])){
			unsigned int index=[fetchResults 
									indexOfObjectIdenticalTo:attribute];
			if(index==NSNotFound){
				index=[fetchResults indexOfObject:attribute];
			}
			if(index==NSNotFound){
				NSString* columnName=[attribute columnName];
				index=1;
				while((index<fcount)&&(NSOrderedSame!=[columnName compare:
						[[fetchResults objectAtIndex:index]columnName]
						options:NSCaseInsensitiveSearch])){
					index+=2;
				}
				if(index>=fcount){
					index=NSNotFound;
				}
			}
			if(index==NSNotFound){
				// They want an attribute that is not in the result set.
				[NSException raise:Oracle8Channel_eAttributeNotInResultSet
						format:NSLocalizedStringFromTableInBundle(@"Unknown attribute=%@, column=%@.",@"Oracle8Channel",Oracle8Utilities_Bundle(),@"Exception raison given when the client expects an attribute that is not in the result set (see: describeResults). [1]:the name of the invalid attribute. [2]:the name of the column of the invalid attribute."),[attribute name],[attribute columnName]];
			}else{
				Oracle8Buffer* buffer=[[[Oracle8Buffer alloc]init]autorelease];
				index--;
				if([buffer defineForColumn:[fetchResults objectAtIndex:index]
						atPosition:1+(index/2)
						inStatement:statement
						asAttribute:attribute]){ //Note:we use the new attrib.
					[buffers addObject:buffer];
				}else{
					// SEE: The error has been reported. Let''s continue.
				}
		    }
		}
		return(buffers);
	}//defineBuffersForAttributes:;


@end //Oracle8Channel(Private);

// -------------------------------------------------------------------------- //

@implementation Oracle8Channel


// NSObject methods:

	-(id)init
	{
		return([self initWithAdaptorContext:nil]);
		// SEE: This is not the designated initializer!
	}//init;


	-(void)dealloc
	{
		if(fetchResults!=nil){
			[fetchResults release];
			fetchResults=nil;
		}
		if(fetchBuffers!=nil){
			[fetchBuffers release]; 
			fetchBuffers=nil;
		}
		if(serviceContext!=nil){
			[self closeChannel];
		}
		[super dealloc];
	}//dealloc;



// EOAdaptorChannel methods:

	-(id)initWithAdaptorContext:(EOAdaptorContext*)anAdaptorContext
	{
		self=[super initWithAdaptorContext:anAdaptorContext];
		if(self!=nil){
			serviceContext=nil;
			statement=nil;
			oracleLanguage=OCI_NTV_SYNTAX;
			fetchResults=nil;
			fetchBuffers=nil;
		}
		return(self);
	}//initWithAdaptorContext:;



	-(BOOL)openChannel
	{
		if([super openChannel]){
			serviceContext=[[(Oracle8Context*)adaptorContext 
								createServiceContext]retain];
			isOpen=(serviceContext!=nil);
			if(isOpen){
				statement=[[O2CI8Statement alloc]init];
			}
			return(isOpen);
		}else{
			return(NO);
		}
	}//openChannel;


	-(void)closeChannel
	{
		[super closeChannel];
		[statement release];
		statement=nil;
		[(Oracle8Context*)adaptorContext 
				disposeServiceContext:[serviceContext autorelease]];
		serviceContext=nil;
	}//closeChannel;


	-(NSArray*)describeResults
	{
		NSMutableArray* result;
		unsigned int    count;
		unsigned int    index;

		if(fetchResults==nil){
			[self makeFetchResults];
		}
		count=[fetchResults count];
		result=[[[NSMutableArray alloc]initWithCapacity:count/2]autorelease];
		for(index=1;index<count;index+=2){
			[result addObject:[fetchResults objectAtIndex:index]];
		}
		return(result);
	}//describeResults;


	-(void)cancelFetch
	{
		[statement fetchRows:0];
		[super cancelFetch];
	}//canceFetch;


	-(NSMutableDictionary*)primaryFetchAttributes:(NSArray*)attributes 
			withZone:(NSZone*)zone
	{
		if(fetchResults==nil){
			[self makeFetchResults];
		}// otherwise, fetch results have been analyzed
		 // by [self describeResults].

		/*
			Note: actually,  the client may change  the attributes set
			      each  time   it  messages  fetchAttributes:   for  a
			      different  row.  Therefore,  we should  rebuild  the
			      fetchBuffers each and  everytime, bare from checking
			      that the  attributes array  did not change  from one
			      invocation to the other (attributes may be a mutable
			      array).

				  The  current  implementation  will expect  the  same
				  attributes array for all the rows.
		*/
		if(fetchBuffers==nil){
			fetchBuffers=[[self defineBuffersForAttributes:attributes]retain];
		}

		if([statement fetchRows:1]){
			NSMutableDictionary* row=[[[NSMutableDictionary allocWithZone:zone]
								initWithCapacity:[attributes count]]autorelease];
			[fetchBuffers oracle8AdaptorMakeThemWorkOn:@selector(fillRow:) 
						  with:row];
			return(row);
		}else{
			return(nil);
		}
		// don't message super!
	}//primaryFetchAttributes:withZone:;


	-(BOOL)evaluateExpression:(NSString*)anExpression
	{
NSLog(@"Oracle8Channel evaluateExpression:\"%@\"\n",anExpression);
		// must not message -[super evaluateExpression:]!
		if(fetchResults!=nil){
			[fetchResults release];
			fetchResults=nil;
			[fetchBuffers release];
			fetchBuffers=nil;
		}
		if([statement prepareStatement:anExpression inLanguage:oracleLanguage]){
			ub4 nIter;
			isFetchInProgress=[statement statementType]==OCI_STMT_SELECT;
			if(isFetchInProgress/*(boolean)*/){
				nIter=0; /*(integer)*/
			}else{
				nIter=1; /*(integer)*/
			}
			if([serviceContext executeStatement:statement iterations:nIter
					rowOffset:0 snapIn:(OCISnapshot*)0 snapOut:(OCISnapshot*)0
					mode:OCI_DEFAULT]){
				return(YES);
			}
		}
		[[adaptorContext adaptor] reportError:[NSString stringWithFormat:
			NSLocalizedStringFromTableInBundle(\
				@"When executing this SQL statement: [%@], obtained this error: %@\n",\
				@"Oracle8Channel",Oracle8Utilities_Bundle(),\
				@"Reporting an error message when executing a SQL statement. [1]:sql statement. [2]:error message."),
			anExpression,
			[[serviceContext errorMessages]	componentsJoinedByString:@"\n"]]];
		return(NO);
	}//evaluateExpression:;


	-(EOModel*)describeModelWithTableNames:(NSArray*)tableNames
	{
		NOT_IMPLEMENTED_YET;
		return([super describeModelWithTableNames:tableNames]);
	}//describeModelWithTableNames:;


	-(NSArray*)describeTableNames
	{
		NOT_IMPLEMENTED_YET;
		return([super describeTableNames]);
	}//describeTableNames;


	-(BOOL)readTypesForEntity:(EOEntity*)anEntity
	{
		NOT_IMPLEMENTED_YET;
		return([super readTypesForEntity:anEntity]);
	}//readTypesForEntity:;


	-(BOOL)readTypeForAttribute:(EOAttribute*)anAttribute
	{
		NOT_IMPLEMENTED_YET;
		return([super readTypeForAttribute:anAttribute]);
	}//reaedTypeForAttribute:;


// Oracle8Channel methods:

	// None.


@end /* Oracle8Channel */


/*** Oracle8Channel.m -- 1999/07/24 04:44:48 -- PJB ***/


