/******************************************************************************
FILE:               Oracle8Context.m
LANGUAGE:           Objective-C
SYSTEM:             GNUstep
USER-INTERFACE:     None
DESCRIPTION

    This class is the GNUstep/db context to the Oracle8 databases.
	Its implementation is based on the OCI v8, and should be compatible
	with, both Oracle87, Oracle88 and future Oracle89 databases.

AUTHORS
    <PJB> Pascal J. Bourguignon <pjb@imaginet.fr>
MODIFICATIONS
    1999/07/24 Creation.
BUGS
    - ...
LEGAL
    Copyright Pascal J. Bourguignon 1999 - 1999
	Copyright Software Builders     1999 - 1999

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/
#include <Oracle8Context.h>
#include <Oracle8Context-ForFriends.h>
#include <Oracle8Channel.h>
#include <Oracle8Utilities.h>

#include <oci.h>
#include <O2CI8UserSession.h>
#include <O2CI8ServerConnection.h>
#include <O2CI8ServiceContext.h>

@implementation Oracle8Context

// NSObject methods:


	-(id)init
	{
		NOT_IMPLEMENTED_YET;
		return(nil);
	}//init;


	-(void)dealloc
	{
		[super dealloc];
	}//dealloc;



// EOAdaptorContext methods:


	-(id)initWithAdaptor:(EOAdaptor*)anAdaptor
	{
		self=[super initWithAdaptor:anAdaptor];
		if(self!=nil){
			if([anAdaptor isKindOfClass:[Oracle8Adaptor class]]){
				serviceContext=nil;
			}else{
				NOT_IMPLEMENTED_YET;
				// raise an exception.
				return(nil);
			}
		}
		return(self);
	}//initWithAdaptor:;


	-(EOAdaptorChannel*)createAdaptorChannel
	{
		return([super createAdaptorChannel]);
	}//createAdaptorChannel;



	-(BOOL)canNestTransactions
	{
		return(NO); // SEE: What can we do for nested transactions.
		// We could probably manage a stack of OCITransaction.
	}//canNestTransactions;


	-(BOOL)primaryBeginTransaction
	{
		if(serviceContext==nil){
			NOT_IMPLEMENTED_YET; // SEE: Report an error: no open channel.
			return(NO);
		}
		if([serviceContext transactionStartBranchTimeout:60 
				flags:OCI_TRANS_NEW|OCI_TRANS_READWRITE]){
			return(YES);
		}else{
			[adaptor reportError:[[serviceContext errorMessages]
							componentsJoinedByString:@"\n"]];
			return(NO);
		}
	}//primaryBeginTransaction;


	-(BOOL)primaryCommitTransaction
	{
		if(serviceContext==nil){
			NOT_IMPLEMENTED_YET; // SEE: Report an error: no open channel.
			return(NO);
		}
		if([serviceContext transactionCommit:OCI_DEFAULT]){
			return(YES);
		}else{
			[adaptor reportError:[[serviceContext errorMessages]
							componentsJoinedByString:@"\n"]];
			return(NO);
		}
	}//primaryCommitTransaction;


	-(BOOL)primaryRollbackTransaction
	{
		if(serviceContext==nil){
			NOT_IMPLEMENTED_YET; // SEE: Report an error: no open channel.
			return(NO);
		}
		if([serviceContext transactionRollback]){
			return(YES);
		}else{
			[adaptor reportError:[[serviceContext errorMessages]
							componentsJoinedByString:@"\n"]];
			return(NO);
		}
	}//primaryRollbackTransaction;


// Oracle8Context methods:

	// None.


@end // Oracle8Context;

// -------------------------------------------------------------------------- //

@implementation Oracle8Context(Private)
	

	-(O2CI8ServiceContext*)createServiceContext
	{
		O2CI8ServerConnection* server;
		O2CI8UserSession*      user;

		if(numOpenChannels>0){
		    NSAssert(serviceContext!=nil,@"When there are already openChannels,"
					@" serviceContext should be non-nil.");
			numOpenChannels++;
			return(serviceContext);
		}
		NSAssert(serviceContext==nil,@"When it's the first openChannels,"
					@" serviceContext should be nil.");
	if([adaptor hasValidConnectionDictionary]){
			NSDictionary* connDict=[adaptor connectionDictionary];
			O2CI8ServerConnection* server=[[O2CI8ServerConnection alloc]init];

			if([server attachToDatabase:
					[connDict objectForKey:Oracle8Adaptor_cdConnectionString]]){
				O2CI8UserSession* user=[[[O2CI8UserSession alloc]init]autorelease];
				[user setUsername:
						[connDict objectForKey:Oracle8Adaptor_cdUsername]];
				[user setPassword:
						[connDict objectForKey:Oracle8Adaptor_cdPassword]];
				serviceContext=[[O2CI8ServiceContext alloc]init];
				if([serviceContext beginSessionWithServer:server forUser:user 
								   withAuthentificationModes:OCI_DEFAULT]){
					transaction=[[O2CI8Transaction alloc]init];
					if(![serviceContext setDVoidP:[transaction handle] 
							attribute:OCI_ATTR_TRANS from:CURRENT_METHOD]){
						[adaptor reportError:[[serviceContext errorMessages]
								componentsJoinedByString:@"\n"]];
					}
					numOpenChannels++;
					return(serviceContext);
				}else{
					[adaptor reportError:[[serviceContext errorMessages]
							componentsJoinedByString:@"\n"]];
					return(nil);
				}
			}
		}else{
			[adaptor reportError:NSLocalizedStringFromTableInBundle(@"The connection dictionary is not valid. Cannot connect to the database.",@"Oracle8Context",Oracle8Utilities_Bundle(),@"Error Message.")];
			return(nil);
		}

	}//createServiceContext;


	-(void)disposeServiceContext:(O2CI8ServiceContext*)aServiceContext
	{
		NSAssert(aServiceContext==serviceContext,@"closing channel should "
				@"pass back the same serviceContext.");
		numOpenChannels--;
		if(numOpenChannels<=0){
			numOpenChannels=0;
			[serviceContext endSession];
			[transaction release];
			transaction=nil;
			[serviceContext release];
			serviceContext=nil;
		}
	}//disposeServiceContext:;

@end // Oracle8Context(Private);


/*** Oracle8Context.m -- 1999/07/24 04:44:49 -- PJB ***/

